library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity plog1_top is
    Port( 
		clk	 : in STD_LOGIC;								-- clk, can also be manipulated by the "step" button
		inc1_n : in STD_LOGIC;								-- Button "inc1"
		ina_n  : in STD_LOGIC_VECTOR (3 downto 0);	-- DIP switch a
		inb_n	 : in STD_LOGIC_VECTOR (3 downto 0);	-- DIP switch b
      led_n	 : out STD_LOGIC_VECTOR(8 downto 0);	-- LED array
		seg1_n : out STD_LOGIC_VECTOR(0 to 6);			-- 7-segment display 1
		seg2_n : out STD_LOGIC_VECTOR(0 to 6)			-- 7-segment display 2
	);
end plog1_top;

architecture structural of plog1_top is

-- Signals to convert negative in-/ outputs
signal inc1 : STD_LOGIC;
signal ina  : STD_LOGIC_VECTOR(3 downto 0);
signal inb  : STD_LOGIC_VECTOR(3 downto 0);
signal led  : STD_LOGIC_VECTOR(8 downto 0);
signal seg1 : STD_LOGIC_VECTOR(0 to 6);
signal seg2 : STD_LOGIC_VECTOR(0 to 6);

-- Signal for task 5
signal cnt_val : STD_LOGIC_VECTOR(3 downto 0);

-- Alias for the buttons
alias btn_a is inc1;
alias btn_b is clk;

begin
	-- Signal conversion form low- to high active
	inc1   <= not(inc1_n);
	ina    <= ina_n;
	inb  	 <= not(inb_n);
	led_n  <= not(led);
	seg1_n <= not(seg1);
	seg2_n <= not(seg2);

	-- Task 1:
	-- Realizes logic "xor" on LED output.
	led_diver0: entity work.led_driver(and_gate)
		port map(
			a_i 	=> btn_a,
			b_i 	=> btn_b,
			led_o => led(0)
		);
	
	-- Task 2:
	-- Enables LED when a button is pressed and 
	-- disables LED when an other button is pressed.
	led_driver1: entity work.led_driver(on_off_2buttons)
		port map(
			a_i 	=> btn_a,
			b_i 	=> btn_b,
			led_o	=> led(1)
		);
		
	-- Task 3:
	-- Toggles the LED when button is pressed. Debounces the button.
	led_driver2: entity work.led_driver(on_off_1button)
		port map(
			a_i 	=> btn_a,
			b_i 	=> clk,
			led_o => led(2)
		);
		
	-- Task 4:
	-- Drives a 7-segment display1 by a 4-bit input value (of the dip switch). 
	-- Shows 'F' if the range is extended.
	seg_driver0: entity work.seg_driver
		port map(
			val_i => ina,
			seg_o	=> seg1
		);
		
	-- Task5:
	-- Drives a 7-segment display2 by a generic counter with asynchronous reset.
	cnt0: entity work.generic_counter
	generic map(
		min_cnt => 0,
		max_cnt => 9)	
	port map(
		rst_i => btn_a, -- async. reset
		clk_i => clk,
		cnt_o => cnt_val
	);
	
	seg_driver1: entity work.seg_driver
	port map(
		val_i => cnt_val,
		seg_o	=> seg2
	);
end structural;

